
 { to obtain forces due to the Moon, the Sun
   by using simple formula
   F_x = - fM_p*(x-x_p)/\Delta^3 - fM_p*x_p/r_p^3
   F_y = - fM_p*(y-y_p)/\Delta^3 - fM_p*y_p/r_p^3
   F_z = - fM_p*(z-z_p)/\Delta^3 - fM_p*z_p/r_p^3
   where
   fM_p - planetocentric gravitational constant in km^3/s^2
   x y z  - the components of the position of the satellite
                with respect to the centre of the Earth
   x_p y_p z_p - the components of the position of the planet
                     with respect to the centre of the Earth
   \Delta^2=(x_p-x)^2+(y_p-y)^2+(z_p-z)^2
   r_p^2=x_p^2+y_p^2+z_p^2
   to obtain
   vector of forces due to Solar radiation pressure
   F_x=S_refl*(AU/delta)^2*(x-x')/delta
   F_y=S_refl*(AU/delta)^2*(y-y')/delta
   F_z=S_refl*(AU/delta)^2*(z-z')/delta
   x  y  z  position of the satellite refered to the Solar system barycentre
   x' y' z' position of the SUn refered to the Solar system barycentre
   delta^2=(x-x')^2+(y-')^2+(z-z')^2
   AU astronomical unit in km
   S_refl special coefficients for each satellite
          in unit km/s^2 ( acceleration ) }

 UNIT UnForceP ;

 INTERFACE

 Uses
   UnConTyp ; { for type TVect3 const AstrUnit }

 Procedure ForceDueToMoon ( teph  : Extended ;       { moment in TDB scale }
                            rvec  : TVect3 ;         { position fixed equa }
                            matr  : TMatr33 ;        { from mean to fix eq }
                            Var  force  : TVect3 ) ; { to add force }

 Procedure ForceDueToSun ( teph  : Extended ;       { moment in TDB scale }
                           rvec  : TVect3 ;         { position fixed equa }
                           matr  : TMatr33 ;        { from mean to fix eq }
                           Var  force  : TVect3 ) ; { to add force }

 Procedure ForceDueToSolPres ( teph  : Extended ;
                               rvec  : TVect3 ;
                               matr  : TMatr33 ;
                               Var  force : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnForFun , { for MultMatrVec procedure }
   UnForMun , { MoonElcPos BrmEquPos procedures }
   UnForMus , { SunNewcomb SimpleSun procedures }
   UnForSop ; { for SatReflCoef global variable }

 Procedure ForceDueToMoon ( teph  : Extended ;       { moment in TDB scale }
                            rvec  : TVect3 ;         { position fixed equa }
                            matr  : TMatr33 ;        { from mean to fix eq }
                            Var  force  : TVect3 ) ; { to add force }
 Var
   ip     : Byte ;
   fmoon  : Extended ;
   v,b,d  : Extended ;
   qq,rr  : Extended ;
   pd,pp  : TVect3 ;
 Begin
   fmoon:=4902.799;          { km^3/s^2 for the Moon }
   MoonEclPos(teph,v,b,d);   { UnForMun ecliptic longitude latitude range }
   BrmEquPos(teph,v,b,d,pd); { UnForMun position in mean equator system }
   MultMatrVec(matr,pd,pp);  { UnForFun position to fixed equator system }
   qq:=Sqr(pp[1])+Sqr(pp[2])+Sqr(pp[3]); { r_p^2 }
   qq:=fmoon/(qq*Sqrt(qq));              { fM_p/r_p^3 }
   For ip:=1 To 3 Do
     pd[ip]:=rvec[ip]-pp[ip];            { x-x_p , y-y_p , z-z_p }
   rr:=Sqr(pd[1])+Sqr(pd[2])+Sqr(pd[3]); { \Delta^2 }
   rr:=fmoon/(rr*Sqrt(rr));              { fM_p/\Delta^3 }
   For ip:=1 To 3 Do
     force[ip]:=force[ip]-rr*pd[ip]-qq*pp[ip]; { to add our forces }
 End;

 Procedure ForceDueToSun ( teph  : Extended ;       { moment in TDB scale }
                           rvec  : TVect3 ;         { position fixed equa }
                           matr  : TMatr33 ;        { from mean to fix eq }
                           Var  force  : TVect3 ) ; { to add force }
 Var
   ip     : Byte ;
   fsoon  : Extended ;
   v,b,d  : Extended ;
   qq,rr  : Extended ;
   pd,pp  : TVect3 ;
 Begin
   fsoon:=1.32712438e11;     { km^3/s^2 for the Sun }
   SunNewcomb(teph,v,b,d);   { UnForMus ecliptic longitude latitude range }
   d:=AstrUnit*d;            { range to km const AstrUnit from UnConTyp }
   BrmEquPos(teph,v,b,d,pd); { UnForMun position in mean equator system }
   MultMatrVec(matr,pd,pp);  { UnForFun position to fixed equator system }
   qq:=Sqr(pp[1])+Sqr(pp[2])+Sqr(pp[3]); { r_p^2 }
   qq:=fsoon/(qq*Sqrt(qq));              { fM_p/r_p^3 }
   For ip:=1 To 3 Do
     pd[ip]:=rvec[ip]-pp[ip];            { x-x_p , y-y_p , z-z_p }
   rr:=Sqr(pd[1])+Sqr(pd[2])+Sqr(pd[3]); { \Delta^2 }
   rr:=fsoon/(rr*Sqrt(rr));              { fM_p/\Delta^3 }
   For ip:=1 To 3 Do
     force[ip]:=force[ip]-rr*pd[ip]-qq*pp[ip]; { to add our forces }
 End;

 { to obtain vector of forces due to Solar radiation pressure
   SatReflCoef special coefficients for each satellite unit km/s^2 }

 Procedure ForceDueToSolPres ( teph  : Extended ;
                               rvec  : TVect3 ;
                               matr  : TMatr33 ;
                               Var  force : TVect3 ) ;
 Var
   i      : Byte ;
   v,b,d  : Extended ;
   tp,rp  : Extended ;
   pr,ps  : TVect3 ;
 Begin
   tp:=teph-0.0057755;       { with the light delay }
   SimpleSun(tp,v,b,d);        { UnForMus ecliptic longitude latitude range }
   d:=AstrUnit*d;            { range to km const AstrUnit from UnConTyp }
   BrmEquPos(tp,v,b,d,pr);   { UnForMun position in mean equator system }
   MultMatrVec(matr,pr,ps);  { UnForFun position to fixed equator system }
   For i:=1 To 3 Do { Satellite refered to the centre of the Sun }
     pr[i]:=rvec[i]-ps[i]; { x-x', y-y', z-z' in km }
   rp:=1/Sqrt(Sqr(pr[1])+Sqr(pr[2])+Sqr(pr[3])); { 1/delta in km }
   For i:=1 To 3 Do  pr[i]:=rp*pr[i]; { (x-x')/delta }
   rp:=SatReflCoef*Sqr(AstrUnit*rp);  { (AU/delta)^2 modul of the acc. }
   For i:=1 To 3 Do  { SatReflCoef global variable }
     force[i]:=force[i]+rp*pr[i]; { acceleration in km/s**2 }
 End;

 END.
