
{ value of tide nominal Love number k_2 = 0.3 }

unit UnForceT ; { geopotential coefficients variations due to tides }

interface

uses
  UnConTyp ; { for types TVect3 TMatr33 }

const
  TideTwoK  = 0.3 ; { value of tide nominal Love number k_2 = 0.3 }

var { variations of geopotential coefficients }
  CTideTwo  : Array [0..2] of Extended ;
  STideTwo  : Array [1..2] of Extended ;

procedure SolidTidePot ( te  : Extended ;  { time TDB }
                         ct  : TMatr33 ) ; { matrix from CRS to TRS }

procedure CoefWithTide ( n , m  : Byte ;  Var  c , s  : Extended ) ; { main }

implementation

uses
  UnForFun , { for procedure MultMatrVec }
  UnForMup , { for the Moon ecliptic position procedure SimpleMoon }
  UnForMus , { for the Sun  ecliptic position procedure SimpleSun  }
  UnForMun ; { proc BrmFixPos from ecliptic of date to fixed equator }

{ for nullo initial value 
  with permanent part of tide
  with solid Earth pole tide }

procedure TidePotNul ;
var
  i  : Byte ;
begin
  for i:=0 to 2 do
    begin
      CTideTwo[i]:=0.0;
      if  i > 0  then  STideTwo[i]:=0.0;
    end;
  { below permanent part of tide }
  CTideTwo[0]:=-(-0.31460)*(4.4228e-8)*TideTwoK*Sqrt(5.0);
end;

{ np number of object NMoon or NSun for example
   p[1..3] terrestrial position of this object
   IERS Conventions 2000 Chapter 6 Geopotential }
  
procedure TidePotAdd ( np  : Byte ;  a  : TVect3 ) ;
var
  g,r,d  : Extended ;
  sf,cf  : Extended ;
  sl,cl  : Extended ;
  sz,cz  : Extended ;
  q,p,x  : Extended ;
begin
  case  np  of
    1 : g:=4902.799/GeoFM;          { km^3/s^2 for the Moon }
    2 : g:=1.32712438e11/GeoFM;     { km^3/s^2 for the Sun }
  end; { case the Moon or the Sun const UnConTyp }
  r:=Sqrt(Sqr(a[1])+Sqr(a[2])+Sqr(a[3])); { range }
  sf:=a[3]/r; { sin(latitude) plus or minus z as argument Legandre polinom }
  cf:=+Sqrt(1-Sqr(sf)); { plus cosinus(latitude) sqrt(1-z^2) for function }
  d:=Sqrt(Sqr(a[1])+Sqr(a[2]));
  cl:=a[1]/d; { cosinus(longitude) cos(lambda) }
  sl:=a[2]/d; { sinus(longitude)   sin(lambda) }
  cz:=Sqr(cl)-Sqr(sl); { cos(2*lambda)=cos^2(lambda)-sin^2(lambda) }
  sz:=2*sl*cl; { sin(2*lambda)=2*sin(lambda)*cos(lambda) }
  d:=GeoR0/r; { GeoR0 in km equator radius of the Earth from UnConTyp }
  x:=d*Sqr(d); { (r_0/r)^3 for n=2 ^(n+1) }
  q:=g*x/5; { (1/(2n+1))*(fM_j/fM_E)*(r_0/r)^(n+1) for n=2 }
  p:=q*(1.5*Sqr(sf)-0.5); { Legandre polinom P_2(z)= 3/2 z^2 - 1/2 , m=0 }
  CTideTwo[0]:=CTideTwo[0]+TideTwoK*p; { n=2 m=0 S_nm=0 real part only }
  p:=3*sf*cf*q; { Legandre function P_21(z)=sqrt(1-z^2)*3*z , m=1 }
  CTideTwo[1]:=CTideTwo[1]      { n=2 m=1 cos(lambda) }
               +TideTwoK*p*cl;  { real part of Love number k_21 }
  STideTwo[1]:=STideTwo[1]      { n=2 m=1 sin(lambda) }
               +TideTwoK*p*sl;  { real part of Love number k_21 }
  p:=3*Sqr(cf)*q; { Legandre function P_22(z)=3(1-z^2) , m=2 }
  CTideTwo[2]:=CTideTwo[2]      { n=2 m=2 cos(2*lambda) }
               +TideTwoK*p*cz;  { real part of Love number k_22 }
  STideTwo[2]:=STideTwo[2]      { n=2 m=2 sin(2*lambda) }
               +TideTwoK*p*sz;  { real part of Love number k_22 }
end;

{ coef. variations for nominal Love numbers }

procedure SolidTidePot ( te  : Extended ;  { time TDB }
                         ct  : TMatr33 ) ; { matrix from CRS to TRS }
var
  u,b,r  : Extended ; { ecliptic longitude latitude range }
  pc,pt  : TVect3 ;   { array[1..3] of extended from UnConTyp }
begin
  TidePotNul ;            { initial nullo and non nullo permanent tide }
  { the Moon refer to the Earth }
  SimpleMoon(te,u,b,r);   { UnForMup ecl. longitude latitude range the Moon }
  BrmFixPos(te,u,b,r,pc); { UnForMun the Moon position to fixed equator CRS }
  MultMatrVec(ct,pc,pt);  { the Moon in the terrestrial system UnForFun }
  TidePotAdd(1,pt);       { geopotential variations due to the Moon }
  { the Sun refer to the Earth }
  SimpleSun(te,u,b,r);    { UnForMus ecl. longitude latitude range the Sun }
  r:=AstrUnit*r;          { range to km const AstrUnit from UnConTyp }
  BrmFixPos(te,u,b,r,pc); { UnForMun the Sun position to fixed equator CRS }
  MultMatrVec(ct,pc,pt);  { the Sun in the terrestrial system UnForFun }
  TidePotAdd(2,pt);       { geopotential variations due to the Sun }
end;

{ to take into account the solid Earth tides
  variations of the geopotential coefficients
  CTideTwo STideTwo are obtained by calling SolidTidePot procedure above }

procedure CoefWithTide ( n , m  : Byte ;  Var  c , s  : Extended ) ;
begin
  if  n > 2  then  Exit ; { no corrections }
  c:=c+CTideTwo[m];
  if  m > 0  then  s:=s+STideTwo[m];
end;

end.
