
 UNIT UnForRoM ; { simple matrix of transformations }

 INTERFACE

 Uses
   UnConTyp ; { for type TMatr33 }

 Procedure FromMeanToFixEquM ( tc  : Extended ;  Var  pf  : TMatr33 ) ;

 Procedure FromFixToTrueEquM ( tc  : Extended ;  Var  pn  : TMatr33 ) ;

 Procedure FromTrueToFixEquM ( tc  : Extended ;  Var  pn  : TMatr33 ) ;

 Procedure FromFixToTerraM ( utc , dut  : Extended ;
                             Var   tdb  : Extended ;
                             Var   ptm  : TMatr33 ) ;

 Procedure FromTerraToFixM ( ptm  : TMatr33 ;  Var  pzm  : TMatr33 ) ;

 Procedure FromFixToTerraP ( utc  : Extended ;
                             xp , yp , dut  : Extended ;
                             Var  tdb  : Extended ; { barycentric time }
                             Var  ptm  : TMatr33 ) ;

 IMPLEMENTATION

 Uses
   UnForFun , { for procedures ForRotMatr , TranspMatr }
   UnForTim , { TransUTCtoTDT for example }
   UnForSit , { EarthRotMatr rotation matrix procedure }
   UnForNut , { function ToGetEpsMean procedure ClcNut }
   UnForPNm ; { procedures ClcPrecMatr ClcNutMatr }

 { tc - moment in scale TDB in modified julian day
   pf - matrix to transform from mean equator to standard equator }

 Procedure FromMeanToFixEquM ( tc  : Extended ;  Var  pf  : TMatr33 ) ;
 Var
   prm  : TMatr33 ; { precession matrix }
 Begin
   ClcPrecMatr(MJD2000,tc,prm); { UnForPnm const from UnCOnTyp }
   TranspMatr(prm,pf);          { UnForFun }
 End;

 { tc - moment in scale TDB in modified julian day
   pn - matrix to transform from standard equator to true equator }

 Procedure FromFixToTrueEquM ( tc  : Extended ;  Var  pn  : TMatr33 ) ;
 Var
   dpsi,deps  : Extended ;
   prec,nutm  : TMatr33 ;
 Begin
   ClcPrecMatr(MJD2000,tc,prec); { UnForPnm precession matrix }
   ClcNut(tc,dpsi,deps) ;  { obliquity radian UnForNut }
   ClcNutMatr(tc,dpsi,deps,ToGetEpsMean(tc),nutm); { UnForPnm UnForNut }
   ToMultMatr(nutm,prec,pn); { from fixed to true equator }
 End;

 { tc - moment in scale TDB in modified julian day
   pn - matrix to transform from true equator to standard equator }

 Procedure FromTrueToFixEquM ( tc  : Extended ;  Var  pn  : TMatr33 ) ;
 Var
   pun  : TMatr33 ;
 Begin
   FromFixToTrueEquM(tc,pun); { matrix from fixed to true equator }
   TranspMatr(pun,pn);        { to transponir UnForFun }
 End;

 { tc  - moment in scale UTC in modified julian day
   ptm - matrix to transform from standard equator to terrestrial system }

 Procedure FromFixToTerraM ( utc , dut  : Extended ;
                             Var   tdb  : Extended ;
                             Var   ptm  : TMatr33 ) ;
 Var
   dt,td  : Extended ;
   pn,pz  : TMatr33 ;   
 Begin
   TransUTCtoTDT(utc,td,dt);  { proc from UnForTim terrestrial time }
   TransTDTtoTDB(td,tdb);     { proc from UnForTim ephemeris time TDB }
   EarthRotMatr(utc,dut,pz);  { proc from UnForSit rotation matrix  }
   FromFixToTrueEquM(tdb,pn); { matrix from fixed  to true  equator }
   ToMultMatr(pz,pn,ptm);     { UnForFun m. fixed  to terra system  }
 End;

 { ptm - matrix to transform from standard equator to terrestrial system 
   pzm - matrix to transform from terrestrial system to standard equator }

 Procedure FromTerraToFixM ( ptm  : TMatr33 ;  Var  pzm  : TMatr33 ) ;
 Begin
   TranspMatr(ptm,pzm); { from UnForFun }
 End;

 { tc  - moment in scale UTC in modified julian day
   xp yp pole position in radian
   dut delta UT in second of time
   ptm - matrix R_2(-x_p)*R_1(-y_p)*R_3(+GAST)*P_nut*P_prec
   to transform from standard equator to terrestrial system }

 Procedure FromFixToTerraP ( utc  : Extended ;
                             xp , yp , dut  : Extended ;
                             Var  tdb  : Extended ; { barycentric time }
                             Var  ptm  : TMatr33 ) ;
 Var
   dt,td  : Extended ;
   pn,pz  : TMatr33 ;  { matrix precession-nutation Earth rotation }
   xm,ym  : TMatr33 ;  { matrix R_2(-x_p) R_1(-y_p) for x pole y pole }
   pe,pp  : TMatr33 ;  { R_3(+GAST)*P_nut*P_prec  R_2(-x_p)*R_1(-y_p) }
 Begin
   TransUTCtoTDT(utc,td,dt);  { proc from UnForTim terrestrial time }
   TransTDTtoTDB(td,tdb);     { proc from UnForTim ephemeris time TDB }
   EarthRotMatr(utc,dut,pz);  { proc from UnForSit rotation matrix  }
   FromFixToTrueEquM(tdb,pn); { matrix from fixed  to true  equator }
   ToMultMatr(pz,pn,pe);      { UnForFun m. fixed  to terra system  }
   ForRotMatr(1,-yp,ym);      { UnForFun matrix R_1(-y_p) }
   ForRotMatr(2,-xp,xm);      { UnForFun matrix R_2(-x_p) }
   ToMultMatr(xm,ym,pp);      { UnForFun matrix R_2(-x_p)*R_1(-y_p) }
   ToMultMatr(pp,pe,ptm);     { UnForFun our matrix to transform }
 End;

 END.
